"""
Gherbal model configuration (HuggingFace-compatible).
"""

from transformers import PretrainedConfig


class GherbalConfig(PretrainedConfig):
    """Configuration class for Gherbal language identification model."""

    model_type = "gherbal"

    def __init__(
        self,
        fasttext_path: str = None,
        labels_path: str = None,
        **kwargs,
    ):
        self.fasttext_path = fasttext_path
        self.labels_path = labels_path

        self.auto_map = {
            "AutoModel": "gherbal_model.Gherbal",
            "AutoConfig": "gherbal_config.GherbalConfig",
        }

        super().__init__(**kwargs)
